#!/usr/bin/python3

import math
import sys

class Node:

    def __init__(self, key, l=None, r=None, p=None):
        self.key = key
        self.l = l
        self.r = r
        self.p = p
        if l != None:
            l.p = self
        if r != None:
            r.p = self

    def find(x, k):
        if x == None or x.key == k:
            return x
        if k < x.key:
            return Node.find(x.l, k)
        else:
            return Node.find(x.r, k)

    def insert(x, k, p=None):
        if x == None:
            return Node(k, None, None, p)
        elif k < x.key:
            x.l = Node.insert(x.l, k, x)
        else:
            x.r = Node.insert(x.r, k, x)
        return x

    def rr(y):
        x = y.l
        b = x.r
        x.r = y
        y.l = b
        p = y.p
        if p != None:
            if p.l == y:
                p.l = x
            else:
                p.r = x
        x.p = p
        y.p = x
        if b != None:
            b.p = y

    def rl(x):
        y = x.r
        b = y.l
        y.l = x
        x.r = b
        p = x.p
        if p != None:
            if p.r == x:
                p.r = y
            else:
                p.l = y
        y.p = p
        x.p = y
        if b != None:
            b.p = x

    def splay_step(x):
        p = x.p
        pp = p.p
        if pp == None:
            if x == p.l:
                p.rr()
            else:
                p.rl()
        else:
            if p == pp.l:
                if x == p.l:
                    pp.rr()
                    p.rr()
                else:
                    p.rl()
                    pp.rr()
            else:
                if x == p.l:
                    p.rr()
                    pp.rl()
                else:
                    pp.rl()
                    p.rl()

    def splay(x):
        if x == None or x.p == None:
            return x
        x.splay_step()
        return x.splay()

    def dump(x, depth=1, parent=None):
        if x == None:
            return
        if x.p != parent:
            print("### BAD PARENT LINK ###")
        Node.dump(x.r, depth+1, x)
        print("    " * depth + str(x.key) + " [" + str(x.size) + "]")
        Node.dump(x.l, depth+1, x)

    def sizes(x):
        s = 1
        if x.l != None:
            x.l.sizes()
            s += x.l.size
        if x.r != None:
            x.r.sizes()
            s += x.r.size
        x.size = s
        x.rank = math.log(s, 2)

    def phi(x):
        if x == None:
            return 0
        else:
            return x.rank + Node.phi(x.l) + Node.phi(x.r)

    def export(x, id=0, parent=-1, ang=0):
        if x == None:
            return id
        print("t({0:d},{1:d},{2:d},{3:d});	// {4:d}".format(parent, ang, x.size, x.key, id))
        pid = id
        id += 1
        id = Node.export(x.l, id, pid, -10)
        id = Node.export(x.r, id, pid, 10)
        return id

    def show(x):
        x.sizes()
        x.dump()
        print()
        print("tn_init();")
        x.export()
        print("tn_draw();")
        print("phi({0:.3f});".format(x.phi()))
        print("", "-" * 60)

mode = ""
if len(sys.argv) > 1:
    mode = sys.argv[1]

if mode == "steps":
    r = None
    for i in range(0, 7):
        r = Node.insert(r, i)
    five = r.find(5)
    r.show()
    five.splay_step()
    r.show()
    five.splay_step()
    r.show()
    five.splay_step()
    five.show()
elif mode == "delete":
    r = None
    r = Node.insert(r, 5)
    r = Node.insert(r, 0)
    r = Node.insert(r, 6)
    r = Node.insert(r, 2)
    r = Node.insert(r, 1)
    r = Node.insert(r, 4)
    r = Node.insert(r, 3)
    r.show()
    r = r.find(2).splay()
    r.show()
else:
    r = None
    for i in range(0, 10):
        r = Node.insert(r, i)
    r.show()
    r = r.find(9).splay()
    r.show()
    r = r.find(0).splay()
    r.show()
    r = r.find(7).splay()
    r.show()
    r = r.find(3).splay()
    r.show()
