/* Společné funkce pro kreslení stromů všeho druhu */

import ads;

pair bst_edge = (0, -1);
pair bst_mini = (0, 0.5);

real tree_node_size = 0.2;
real tree_sub_size = 0.23;
real tree_sub_aspect = 1;
pair tree_label_offset = (0,0);

void tree_init(pair g[])
{
	current_graph = g;
	vertex_size = tree_node_size;
}

void tree_node(int i, string name = "", int mode=1)
{
	vertex(i, mode);
	label("\strut $" + name + "$", current_graph[i] + tree_label_offset);
}

void tree_num(int i, int x, int mode=1)
{
	vertex(i, mode);
	label("$" + (string) x + "$", current_graph[i] + tree_label_offset);
}

void tree_num_inverse(int i, int x)
{
	vertex(i, 0);
	label("$" + (string) x + "$", current_graph[i] + tree_label_offset, white);
}

void tree_ext(int i, real size=0.5, int mode=1)
{
	pair p = current_graph[i];
	path v = shift(p - size*(vertex_size, vertex_size)) * scale(size*2*vertex_size) * unitsquare;
	draw_in_mode(v, mode);
}

void tree_sub(int i, string name = "")
{
	pair p = current_graph[i];
	real vsize = tree_sub_size * tree_sub_aspect;
	draw(p -- p + (-1.5*tree_sub_size, -3*vsize) -- p + (1.5*tree_sub_size, -3*vsize) -- cycle);
	label("\ninerm " + name, p + (0, -2*vsize));
}

void tree_elliptic_node(int i, string name = "", real elong=1, int mode=1)
{
	pair p = current_graph[i];
	real h = vertex_size;
	real w = elong*vertex_size;
	draw_in_mode(
		arc(p-(w,0), h, 90, 270) --
		arc(p+(w,0), h, -90, 90) --
		cycle,
		mode
	);
	label("\strut $" + name + "$", p - (0, 0.02) + tree_label_offset);
}

void tree_edge(int i, int j, pen p = thin)
{
	draw(current_graph[i] -- current_graph[j], p);
}

/* AVL stromy */

void lbracket(real x, real ytop, real ybot, string name, real lpos = 0.5)
{
	pair t = (x, ytop);
	pair b = (x, ybot);
	draw(t -- t-(0.1,0) -- b-(0.1,0) -- b);
	label("$" + name + "$", point(t--b, lpos), 2*W);
}

void rbracket(real x, real ytop, real ybot, string name, real lpos = 0.5)
{
	pair t = (x, ytop);
	pair b = (x, ybot);
	draw(t -- t+(0.1,0) -- b+(0.1,0) -- b);
	label("$" + name + "$", point(t--b, lpos), 2*E);
}

real br_top_vertex(int i)
{
	return current_graph[i].y + 1.2*vertex_size;
}

real br_bot_sub(int i)
{
	return current_graph[i].y - 3.5*tree_sub_size;
}

void tree_sub_ht(int i, string name = "", string ht)
{
	tree_sub(i, name);
	label("$" + ht + "$", current_graph[i] + 3*tree_sub_size*tree_sub_aspect*S, S);
}

/* (a,b)-stromy */

void ab_edge(int i, int j, real delta = 0, pen p = thin)
{
	draw(current_graph[i] + (delta,0) -- current_graph[j], p);
}

/* LLRB stromy */

pen rb_red = cmyk(0, 0.9, 0.7, 0.2) + colorless(halfthick);
pen rb_black = gray(0) + halfthick;
pen rb_any = agray + halfthick;

/* Úplné binarní stromy */

pair [] complete_binary_tree(real width, real spread, int depth)
{
	pair u[];
	u[0] = (0,0);
	u[1] = (0,0);
	int r = 1;
	int w = 2;
	for (int i=1; i<depth; ++i) {
		real d = width / 2**i;
		for (int j=0; j<2**(i-1); ++j) {
			u[w] = u[r] - (d,spread);
			u[w+1] = u[r] + (d,-spread);
			w += 2;
			++r;
		}
	}
	return u;
}

/* Pohodlnější konstrukce stromů */

struct tnode {
	int id;
	pair pos;
	pair direction;
	tnode parent;
	tnode sons[];
	int size;
	int mode;		// Drawing mode
	void draw_vertex(tnode w) { vertex(w.id, mode=this.mode); }
	void draw_edge(tnode w) { if (w.parent != null) tree_edge(w.parent.id, this.id); }
};

tnode tnodes[];
pair tn_pos[];
pair tn_ref;
real tn_edge_len = 1;

void tn_init(pair refpos=(0,0))
{
	tnodes = new tnode[];
	tn_pos = new pair[];
	tn_ref = refpos;
	tree_init(tn_pos);
}

tnode tn_add(int parent_id, real angle)
{
	tnode v;
	v.id = tnodes.length;
	v.sons = new tnode[];
	v.mode = v_white;
	tnodes.push(v);
	pair d = (0,1) * dir(angle);
	d = d * (-tn_edge_len/d.y);
	if (parent_id < 0) {
		v.pos = tn_ref;
	} else {
		v.parent = tnodes[parent_id];
		v.parent.sons.push(v);
		v.pos = v.parent.pos + d;
	}
	tn_pos.push(v.pos);
	return v;
}

tnode tn_add(tnode parent, real angle)
{
	return tn_add(parent != null ? parent.id: -1, angle);
}

void tn_draw()
{
	for (tnode v: tnodes)
		v.draw_edge(v);
	for (tnode v: tnodes)
		v.draw_vertex(v);
}
