-- Namespace

ebook = {}

-- Debugging functions

function ebook.enable_debug()
	ebook.debug_mode = 1
	ebook.debug = function(msg)
		print(msg)
	end
	ebook.debug_nodes = function(head)
		nodetree.analyze(head)
	end
end

function ebook.disable_debug()
	ebook.debug_mode = nil
	ebook.debug = function(msg) end
	ebook.debug_nodes = function(head) end
end

ebook.disable_debug()

function ebook.warn(msg)
	texio.write_nl("WARNING: " .. msg .. "\n")
end

-- Configuration

ebook.config = {}

function ebook.load_config(filename)
	local fn = kpse.find_file(filename, true)
	if not fn then
		tex.error("Cannot find configuration file " .. filename)
	end

	local f = io.open(fn, "r")
	if not f then
		tex.error("Cannot read configuration file " .. fn)
	end

	local config = ebook.config
	local section = nil
	local lino = 0

	while true do
		lino = lino + 1
		local line = f:read()
		if not line then break end
		if string.match(line, "^%s*$") or string.match(line, "^[#;]") then
			-- Blank lines and comments are ignored
		else
			local sec = string.match(line, "^%[([0-9a-zA-Z_-]*)%]$")
			if sec then
				if not config[sec] then
					config[sec] = {}
				end
				section = config[sec]
			else
				local key, value = string.match(line, "^([0-9a-zA-Z_-]+)=(.*)$")
				if key and section then
					section[key] = value
				else
					tex.error("Parse error in " .. filename .. ", line " .. lino)
				end
			end
		end
	end

	f:close()

	if ebook.get_config_bool('general', 'debug', 0) then
		ebook.enable_debug()
	end

	if ebook.debug_mode then
		for s, sect in pairs(config) do
			for k, v in pairs(sect) do
				ebook.debug(string.format("Config [%s] %s=%s", s, k, v))
			end
		end
	end
end

function ebook.get_config(s, k, default)
	if ebook.config[s] and ebook.config[s][k] then
		return ebook.config[s][k]
	else
		if default then
			return default
		else
			tex.error(string.format("Missing mandatory config item %s.%s", s, k))
		end
	end
end

function ebook.get_config_bool(s, k, default)
	if default then
		default = tostring(default)
	end
	local v = ebook.get_config(s, k, default)
	if v == "0" or v == "no" or v == "false" then
		return nil
	elseif v == "1" or v == "yes" or v == "true" then
		return 1
	else
		tex.error(string.format("Invalid boolean config item %s.%s = %s", s, k, v))
	end
end

-- Access to configuration from TeX

function ebook.tex_get_config_bool(s, k, default)
	local v = ebook.get_config_bool(s, k, default)
	if v then
		tex.write("1")
	else
		tex.write("0")
	end
end
